/*
 * coordinatesystem.hpp
 *
 *  Created on: Oct 2, 2017
 *      Author: julien
 */

#ifndef SRC_COORDINATE_SYSTEM_COORDINATESYSTEM_HPP_
#define SRC_COORDINATE_SYSTEM_COORDINATESYSTEM_HPP_

#define SWITCHED_AXIS 1

#include "../trajectory_planner/Lane/Lane.hpp"
#include "../../OpenCV_includes.hpp"

class Coordinate_System {

	private:

		int coordinate_length; // complete width of the coordinate system
		int range; // coordinate length/2
		double length_ratio; // how many pixels are one length unit
		int marking_rate; // at what pixel rate should the markings occur

		Mat system; // this is the system that will be used to represent data and graphs

	public:


		/**
		 * initializes the coordinate system. The system will be quadratic in form.
		 *
		 * @param int system_length : system length is the pixel length of the output
		 * 							  A value of 200 gives you a 200x200 square for outputting
		 * 							  graphes and points
		 * @param double length_ration : this determines how many pixels will be used to represent
		 * 								 one length unit. So for a value of 200, 200 pixels will be
		 * 								 used to represent a length of one, and for a value of 0.2
		 * 								 one pixel will 5 length units
		 * @param int marking_rate : defines where the markings will be on the coordinate system
		 * 							 at a value of two, there will be a marking every second
		 * 							 length unit
		 */
		Coordinate_System(int system_length, double length_ratio, int marking_rate);



		/**
		 * transforms a point in standard pixel coordinates (columns and rows)
		 * into central ego coordinates
		 * This is for plotting graphs and functions. (Mostly for testing algorithms and debugging)
		 * From:			Into:
		 * +--------->		       ^x
		 * |		x		       | (this is not in SI-Units)
		 * |				y	   |
		 * |				<------+-------
		 * |y					   |
		 * v					   |
		 */
		Function::pPoint pixel2central_coord(Point pixel_point);

		/**
		 * does the reverse of pixel2central_coord. This function is used more often, than the other
		 */
		Point central_coord2pixel(Function::pPoint central_point);

		/*
		 * returns a Mat where the given polynomial function is plotted onto the coordinate system
		 * This function basically draws the function into the given coordinate system.
		 * be careful at choosing the coordinate system
		 *
		 * @param vector<double>* poly : The function that will be plotted. Make sure that
		 * 								 this is a polynomial function and powers are equal
		 * 								 to their index.
		 * @param Mat* output : This is the coordinate system that will be drawn upon.
		 * 						output needs to be a coordinate system given by the function
		 * 						"get_Coordinate_System"
		 * @param Scalar color : Choose a BGR color to represent the function
		 */
		void plot_f(vector<double>* poly, Mat* output, Scalar color);

		/**
		 * plots the given polynomial function onto the coordinate system within the given
		 * range [x0, x1].
		 * For example: if x0 = -2 and x1 = 2, then the function will be drawn between -2 and 2
		 *
		 * @param vector<double>* poly : The function that will be plotted. Make sure that
		 * 								 this is a polynomial function and powers are equal
		 * 								 to their index.
		 * @param Mat* output : This is the coordinate system that will be drawn upon.
		 * 						output needs to be a coordinate system given by the function
		 * 						"get_Coordinate_System"
		 * @param Scalar color : Choose a BGR color to represent the function
		 * @param double x0 : this is the start point for the plotting range
		 * @param double x1 : this is the end point for the plotting range
		 */
		void plot_f_in_range(vector<double>* poly, Mat* output,Scalar color,
						double x0, double x1);

		/**
		 * This plots a given 2D vector function onto a given coordinate system
		 * inside the given range [start_t, end_t] where start_t < end_t
		 *
		 * @param Mat* output : This is the coordinate system that will be drawn upon.
		 * 						output needs to be a coordinate system given by the function
		 * 						"get_Coordinate_System"
		 * @param Function::vec_func* v : this is a function that will be plotted onto the coordinate
		 * 								  system. Make sure, that poly_x and poly_y are not empty
		 * @param double start_t : the starting parameter for both x(t) and y(t) for plotting
		 * @param double end_t : the end parameter for both x(t) and y(t) for plotting
		 * @param double granularity : determines how finely grained the picture will look.
		 * 							   a higher value will look more chunky, but performs faster in
		 * 							   terms of drawing. A smaller number (like 0.1) will look
		 * 							   smoother but will take longer to draw.
		 * @param Scalar color : Choose a BGR color to represent the function
		 */
		void plot_vec_func(Mat* output,
						   Function::vec_func* v,
						   double start_t,
						   double end_t,
						   double granualirity,
						   Scalar color);

		void plot_vec_func(Mat* output, Function::vec_func* v, Scalar color);

		/**
		 * draws a collection of Points onto a given coordinate system
		 *
		 * @param Mat* output : This is the coordinate system that will be drawn upon.
		 * 						output needs to be a coordinate system given by the function
		 * 						"get_Coordinate_System"
		 * @param vector<Function::pPoint> points : the collection of Points that will be drawn
		 * 											into the coordinate system. These Points
		 * 											need to be in central coordinate format
		 * @param Scalar color : Choose a BGR color to represent the function
		 */
		void draw_points(Mat* output, vector<Function::pPoint>* points, Scalar color);

		/*
		 * returns just the empty coordinate system as mat into the output-pointer,
		 * ready to be drawn upon. This is also a handy function to clear an existing
		 * coordinate system
		 */
		void get_Coordinate_System(Mat* output);

		/**
		 * plots vector (array) of vector functions. This is used for plotting the ego lane
		 * (as in outputted by the lane module)
		 */
		void plot_ego_lane(Mat* output, vector<Function::vec_func>* ego_lane);


		/**
		 * TODO
		 */
		void plot_lane_points(Mat* output,
							  vector<vector<vector<Function::pPoint>>>* lane_vector);

		/**
		 * TODO
		 */
		void plot_lane_vecs(Mat* output, vector<vector<Function::vec_func>>* lane_line_vecs);


};

#endif /* SRC_COORDINATE_SYSTEM_COORDINATESYSTEM_HPP_ */
